.\" $Id: keeper.man,v 1.31 1998/03/27 22:52:43 esr Exp $
.TH KEEPER 1 "6 Mar 1997"
.AT 3
.SH NAME
keeper \- archivist's assistant for FTP & WWW archive sites
.SH SYNOPSIS
.B keeper [-dnw] [-p dir] [-r dir] [-s pref] 
.PP
.SH THEORY
.PP
\fIKeeper\fR is an archivist's assistant for large Internet-accessible
software repositories.  Such sites need to be able to (a) file
submissions into an extensible topic tree, (b) associate descriptive
meta-information with each submission, and (c) maintain FTP and WWW
indices which make the archive contents and metainformation readily
browseable and searchable.
.PP
\fIKeeper\fR assumes that your archive setup is roughly like
Sunsite's.  That is, you have an FTP hierarchy that is WWW-accessible,
divided into a hierarchy of topic directories (the `topic
tree'). There is an incoming directory where submissions can be
dropped.  You need to match submissions with older versions, and
replace older submissions when appropriate while preserving as much
meta-information as possible.  You want to be able to automatically
maintain parallel FTP and HTML index files for each topic directory.
Some directories in the topic tree are actually mirror points for
archives on other sites and must be treated as "read-only".
.PP
The topic knowledge in a keeper-maintained archive is expressed in two
ways: (1) the arrangement of the topic directories, and (2) !INDEX
files.  An !INDEX file contains one line of meta-information
(typically description info) for each non-document file in its
directory, including subdirectories.  An !INDEX file may also contain
a long-format description of its directory.  !INDEX files are directly
browseable by FTP and are used to automatically generate both long-
and short-format HTML index pages and FTP-able README files.
.PP
\fRKeeper\fR makes one other assumption about user-supplied
meta-information; that every submission comes accompanied with an LSM
(Linux Software Map) entry.  This is important for two reasons.
First, \fIkeeper\fR relies on the Author and Maintainer fields in the
LSM to generate email notifications when the status of a package
changes.  Second, requiring an LSM ensures that archive users will
always be easily able to get at package descriptions written by the
package maintainers themselves (as opposed to one-liners written by an
overworked archivist!)
.PP
.SH DESCRIPTION
.PP
\fIKeeper\fR itself is an dispatcher program resembling a
specialized shell.  You will normally use it in interactive mode, but
it has some batch options (documented below).
.PP
In the interactive mode, \fIkeeper's\fR view of the world is organized
around the idea of a "filegroup", a collection of archive and document
files that should be treated as a unit for archiving purposes.
\fIKeeper\fR is a tool for picking up filegroups, matching them
against other filegroups, and moving them between places in the
archive tree (initially from the incoming-submissions directory to a
topic directories and possibly later between topic directories).
\fIKeeper\fR also supports editing the topic tree itself.
.PP
The commands of interactive \fIkeeper\fR are fully documented in its
own on-line help; type `?' at the main prompt to get started.
.PP
\fIKeeper\fR has proper multi-user interlocks, so it can be used by
several collaborating archive maintainers simultaneously.  Keeper
creates directories and files with 775 permissions; if all maintainers
are members of the archive ownership group specified in the
configuration section, they will be able to modify the archive.
.PP
The -f option may be used to read in a file of Perl variable settings
to override those in the wired-in configuration section.
.PP
\fIKeeper\fR has batch modes triggered by command-line options:
.TP 5
-d 
Dump keeper's configuration globals.  This can be used to check on the
effects of configuration files.
.TP 5
-n
Report name conflicts.  Report all cases in which the same stem occurs
in multiple directories in the archive tree (excluding directories 
beneath mirror points, and not counting directories which are symlinks).
.TP 5
-r <dir>
Rebuild the subtree with <dir> at its top (<dir> must be given
relative to the archive top).  To rebuild the whole archive, do `-r /'.
A rebuild checks all !INDEX files. prompts for missing descriptions,
and regenerates both long and short-form HTML index files in the
subtree.
.TP 5
-w
Rebuild the INDEX.whole.gz file.  This is normally done at the end of
each keeper run.
.TP 5
-s
Stem.  Extract and display the stem of the filename argument; that is,
the prefix keeper would use to match it against other filenames.
(This option is intended for debugging and nay be removed from future
versions.)
.TP 5
-p
Policy check.  Check all LSMs and list every copying policy that
doesn't match one of the following known license types: 
.TP 10
GPL
GNU General Public License
.TP 10
GPL 2.0
GNU General Public License, version 2.0
.TP 10
BSD
Berkeley Regents copyright (used on BSD code)
.TP 10
X Consortium
MIT X Consortium license (rather like BSD's)
.TP 10
Artistic License
Same terms as Perl Artistic License
.TP 10
shareware
copyrighted, no restrictions, contributions solicited
.TP 10
FRS
copyrighted, freely redistributable
.TP 10
PD
placed in public domain
.PP
At the end of the list, utter some statistics on numbers and
frequencies of license types.  The statistics lines are preceded by
a single hash.
.PP
.SH FILE PREFIXES AND MATCHING SEMANTICS
.PP
Keeper tries to extract the longest possible alphabetic prefix from
each filename it looks like; this prefix is considered to identify the
file's package.  Prefixes may contain dashes and underscores, but not
dots. 
.PP
Keeper can identify files with GNU-style names (that is, containing a
dash followed by a GNU-style version number).  A file with a GNU-style
name may have digit characters in its prefix (otherwise the prefix
stops on the first digit); also, keeper knows how to extract the
version number part of a GNU-style name.
.PP
Filegroups in incoming are collected by common-prefix match.  When you
do a `guess' command to try to find a match for a current filegroup
filegroup in the archive, the search is actually done by the same
prefix match.
.PP
When keeper finds a filegroup, it checks to see if the filegroup match
is `exact'.  A match is exact if (a) source and target filegroup
contain the same number of filegroups, (b) source and target files can
be paired off such that the names in each pair are either identical or
are GNU-style and only differ in version number, and (c) the
maintainer e-mail address lists extracted from source and target LSM
files are identical.
.PP
When a filegroup match is exact, keeper assumes that existing
descriptions for old files can be applied to the new ones.
.PP
.SH PACKAGE NAME POLICY
If the global $strictnames is set, keeper requires all filenames in
submitted packages to be GNU-style (except LSMs which may omit the
version number).  Packages which contain non-conforming names will
be rejected when $strictnames is on (with notice to the maintainers).
.PP
If $strictnames is off, non-conforming packages will be accepted but
the submitters will receive a notice nudging them to rename their
files appropriately.
.PP
If the global $strictcopy is set, keeper will reject any package that
does not have one of the recognized free-software copyrights attached
to it.
.PP
If $strictcopy is off, non-conforming packages will be accepted but
the submitters will receive a notice nudging them to re-license their
packages in a standard way.
.PP
In either case submitters will be referred to the URL given in
$nameurl for details.
.PP
.SH INDEX AND DERIVED FILES
.PP
The maintainer-supplied information in a keeper archive is contained
in master index files, one per topic directory.  Each of these files
is named !INDEX (this name is chosen because it's quite unlikely to
be part of a package, and so it will sort to the top in an FTP
listing).
.PP
Each !INDEX file consists of four parts in order from the top: a header
line, an optional long desacription section, a delimiter line of
dashes, and a file-descriptions part.
.PP
The first line is used to identify the file and stamp it as
keeper-generated.
.PP
Any text after the first line but before before the line of dashes in
an !INDEX file is treated as a long directory description.  It is
preserved across index rebuilds, inserted in the header portion of the
long-form HTML index when that is rebuilt.  It is also used to
generate an FTPable README whenever there is an index rebuild.  If a
recognizable URL occurs in this text (that is, a word consisting of a
colon-terminated service prefix followed by //), it will be turned
into a live link in the HTML index.
.PP
After the line of dashes, each line is presumed to consist of a
filename, followed by whitespace, followed by one line of description.
There should be such a description for every package in the directory.
.PP
When an archive action (such as moving a package into a directory)
triggers an index rebuild, the contents of these sections is
reformatted into three different derived indexes; a long-form HTML
index called !INDEX/html, a short-form HTML file called
!INDEX.short.html, and an FTPable README file.
.PP
.SH ACTION NOTICES
.PP
\fIKeeper\fR tries to email a rejection notice with description of the
problems whenever you process an incoming filegroup with corrupt
archives or a missing or invalid LSM file (keeper insists omn LSM
version 3).  It will prompt for an author address if it can't get one
from an LSM.  You may be able find the author by looking for older
versions in the archive or with a web search.
.PP
\fIKeeper\fR tries to mail out action notices to the maintainers
whenever files are zapped out of a file group.  It prompts for a
one-line reason.  If it can't parse an email contact list out of LSM
files in the group, it will skip this step.
.PP
\fIKeeper\fR tries to mail out action notices to the maintainers
whenever a package is moved (either placed from the incoming directory
or moved withing the topic tree).  If it can't parse an email contact
list out of the LSM file it will skip this step.
.PP
\fIKeeper\fR tries to mail out action notices to the maintainers
whenever a package file description is changed. If it can't parse an
email contact list out of the LSM file it will skip this step.
.PP
\fIKeeper\fR can send a form letter complaining about a package's name
when asked to with the `yell' command.  Under some circumstances (when
it detects non-GNUish names) it will yell automatically. See the
on-line help for the `yell' command for details.
.PP
Whenever \fIkeeper\fR sends out an email notice, the relevant
directory is checked against a watch list and possibly copied to
interested archive maintainers.  See the description of the watchlist
file below.
.PP
KEEPER AND SYMBOLIC LINKS
Keeper-maintained archives may contain symbolic links.  Symbolic
links between directories may be used to allow a topic category to
live under multiple archive categories.  
.PP
Keeper tries to rewrite any symbolic links that should change when a
source or target directory is moved.
.PP
When a filegroup deletion or replacement deletes the target of a
symbolic link, the user is warned that this has occurred.
.PP
KEEPER AND MIRROR POINTS
.PP
\fIKeeper\fR can be given a list of configuration files which are
assumed to be in the format used by the Perl `mirror' package.  These
will be scanned for local mirror points.
.PP
In index files, a mirror point will be flagged with `/&' on the 
directory entry in its parent. 
.PP
Mirror points are otherwise made invisible in \fIkeeper\fR; you cannot
move files into or out of them, you cannot change directory to them,
and \fIkeeper\fR will not try to match incoming filenames in them
.PP
An attempt to delete a mirror point directory (or any directory
above it) will be detected and rejected.
.PP
An attempt to move a mirror point directory (or any directory
above it) will be detected and rejected. 
.PP
These two files are automatically generated in the archive's root directory.
.SH FILES
.TP 5
NEW
Each dispatch of a file group from the incoming directory to a topic
directory is logged here.
.TP 5
NEW.html
HTMLized version of NEW intended to be periodically put at an
accessible URL in the archive.
.TP 5
INDEX.whole.gz
The concatenation of all the plain index files, compressed for fast
downloading.
.PP
All these files are in the `shared' directory configured near the
head of the source.  All are updated automatically.
.TP 5
LOG
Each edit of the topic tree, and each filegroup move internal to the
topic tree, is logged here.
.TP 5
watchlist
A list of watch-list assignments, one per line.  Each consists of a
first whitespace-delimited field which is interpreted as an email
address.  Subsequent whitespace-delimited fields are Perl regexps to be
matched against directory names.  Each time an email notice is sent,
the directory it is relevant to is checked against the watch regexps
and blind-copied to any emal address with a matching regexp.
.TP 5
LOCK
Exclusion lock file.
.TP 5
LOCK.ASSERTED
Link to exclusion lock file.  Only exists when some copy of keeper has
asserted a lock.
.PP
The following files may be used to set keeper's internal globals:
.TP 5
/etc/keeper.rc
This site-wide configuration file is read first.
.TP 5
~/.keeper
This per-user configuration is read second.
.PP
The file specified by the -f option (if specified) is read last.
.SH BUGS
.PP
\fIKeeper\fR will think that an !INDEX file containing no description
lines and no header test is not in keeper format, even if it has a
correct header.
.PP
In principle, when \fIkeeper\fR can detect that a directory is a
symlink to a mirror point elsewhere, it could permit the link to
be moved or deleted while still suppressing other operations.  This
will be supported in future versions.
.SH AUTHOR
Eric S. Raymond <esr@thyrsus.com>.  Uses some code originated by Erik
Troan <ewt@redhat.com>.
